library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-------------------------------------------------------------------------------
-- Transmitter unit
-------------------------------------------------------------------------------
entity TxUnit is
  port (
     Clk    	: in  Std_Logic;  -- Clock signal
     Reset_n  	: in  Std_Logic;  -- Reset input, active low
     Load   	: in  Std_Logic;  -- Load transmit data
     TxD    	: out Std_Logic;  -- RS-232 data output
     TRegE_n  	: out Std_Logic;  -- Tx register empty, active low
     TBufE_n  	: out Std_Logic;  -- Tx buffer empty, active low
     DataO  	: in  Std_Logic_Vector(7 downto 0)); 
end entity;

architecture Behaviour of TxUnit is
  signal TBuff    : Std_Logic_Vector(7 downto 0); -- transmit buffer
  signal TReg     : Std_Logic_Vector(7 downto 0); -- transmit register
  signal tmpTRegE_n : Std_Logic;
  signal tmpTBufE_n : Std_Logic;
begin
  process(Clk)
    variable BitCnt : Integer range 0 to 9;
  begin
     if Rising_Edge(Clk) then
        if Reset_n = '0' then
           tmpTRegE_n <= '0';
           tmpTBufE_n <= '0';
           TxD <= '1';
           BitCnt := 0;
        else
           if Load = '1' then
              if tmpTRegE_n = '0' then
                 TReg <= DataO;
                 tmpTRegE_n <= '1';
              else
                 TBuff <= DataO;
                 tmpTBufE_n <= '1';
              end if;
           end if;
           if tmpTRegE_n = '1' then
              case BitCnt is
                 when 0 =>
                    TxD <= '0';
                    BitCnt := BitCnt + 1;
                 when 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 =>
                    TxD <= TReg(0);
                    TReg <= '1' & TReg(7 downto 1);
                    BitCnt := BitCnt + 1;
                 when 9 =>
                    TxD <= '1';
                    TReg <= '1' & TReg(7 downto 1);
                    BitCnt := 0;
                    if tmpTBufE_n = '1' then
                       TReg <= TBuff;
                       tmpTBufE_n<='0';
                    else
                       tmpTRegE_n <= '0';
                    end if;
                 when others => null;
              end case;
           end if;
        end if;
     end if;
  end process;

  TRegE_n <= tmpTRegE_n;
  TBufE_n <= tmpTBufE_n;
end Behaviour;
